' DISCLAIMER:
' ALL SOURCE CODE IN THIS PROJECT IS PROVIDED BY ALIBRE INC. FOR
' DEMONSTRATION PURPOSES ONLY AND ANY EXPRESSED OR IMPLIED WARRANTIES
' ARE DISCLAIMED. IN NO EVENT SHALL ALIBRE INC. BE LIABLE FOR ANY
' DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
' DAMAGES ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE.



Public Class frmPartInsertAssmSave
    Private hook As IAutomationHook      ' Holds Alibre Automation hook object
    Private rootObj As IADRoot                  ' Holds Alibre Root object
    Private objADOccurrence As IADOccurrence    ' Holds Occurrence object
    Private DesignSession As IADDesignSession   ' Holds Alibre Design Session object
    Private objPartSession As IADPartSession    ' Holds Alibre Part Session object
    Private objSession As IADSession            ' Holds Alibre Session Object

    Private Sub btnInsert_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInsert.Click
        On Error GoTo Error_Trap

        Dim objAssmSession As IADAssemblySession    ' Holds Alibre Assembly Session Object
        Dim objADRootOccurrence As IADOccurrence    ' Holds Root Occurrence of the Assembly
        Dim objADOccurrences As IADOccurrences      ' Holds all Occurrences of the Assembly
        Dim destinationString As String             ' Holds the location where the File gets Saved
        Dim flag As Boolean
        flag = True

        If (rootObj Is Nothing) Then                'Exit if for some reason an instance of Alibre Design could not be found
            Exit Sub
        End If

        If (rootObj.Sessions Is Nothing) Then       'Exit if for some reason an Alibre Session Object could not be found
            Exit Sub
        End If

        If (rootObj.Sessions.Count > 0) Then        ' If there is atleast one workspace open
            For Each objSession In rootObj.Sessions
                If ((objSession.SessionType = ADObjectSubType.AD_ASSEMBLY) And (flag = True)) Then  ' If there is atleast one Assembly open
                    objAssmSession = objSession                                 ' part is inserted into that assembly
                    flag = False

                    lblStatus.Text = "Inserting Part into the Assembly..."

                    ' Set the Object Session to be the Assembly's session
                    objSession = objAssmSession

                    ' Get Root Occurrence from Assembly Session
                    objADRootOccurrence = objAssmSession.RootOccurrence()

                    ' Get Occurrences collection from Root Occurance
                    objADOccurrences = objADRootOccurrence.Occurrences()

                    ' Holds Geometry Factory
                    Dim objADGeometryFactory As IADGeometryFactory

                    ' Get Geometry Factory from Session object
                    objADGeometryFactory = objSession.GeometryFactory

                    ' Holds Transformation Array Data
                    Dim adblTransformationArrayData(15) As Double

                    ' Populate the Transformation Array with the following Data for Back View
                    '   1   0   0   0
                    '   0   1   0   0
                    '   0   0   1   0
                    '   0   0   0   1

                    adblTransformationArrayData(0) = 1
                    adblTransformationArrayData(1) = 0
                    adblTransformationArrayData(2) = 0
                    adblTransformationArrayData(3) = 0

                    adblTransformationArrayData(4) = 0
                    adblTransformationArrayData(5) = 1
                    adblTransformationArrayData(6) = 0
                    adblTransformationArrayData(7) = 0

                    adblTransformationArrayData(8) = 0
                    adblTransformationArrayData(9) = 0
                    adblTransformationArrayData(10) = 1
                    adblTransformationArrayData(11) = 0

                    adblTransformationArrayData(12) = 0
                    adblTransformationArrayData(13) = 0
                    adblTransformationArrayData(14) = 0
                    adblTransformationArrayData(15) = 1

                    ' Holds Transformation
                    Dim objADTransformation As AlibreX.IADTransformation

                    ' Create Transformation
                    objADTransformation = objADGeometryFactory.CreateTransform(adblTransformationArrayData)

                    ' Add an Empty Part as Occurrence
                    objADOccurrence = objADOccurrences.AddEmptyPart("BlockWithHole", False, objADTransformation)

                    ' Set Design Session to be the empty Part's Design Session that was just added to the assembly
                    DesignSession = objADOccurrence.DesignSession

                    ' Set Part Session to be the empty Part's Design Session that was just added to the assembly
                    objPartSession = DesignSession

                    ' Call to CreateFeatures method to add features to the empty part inserted into the assembly
                    CreateFeatures()

                    lblStatus.Text = "Part inserted successfully into " & objSession.Name

                    'Saves the Assembly with the Part to the location specified
                    lblStatus.Text = "Saving assembly on C:\ Drive..."
                    destinationString = "C://"
                    Call objSession.SaveAs(destinationString, objSession.Name)
                    lblStatus.Text = "Assembly is saved successfully on C:\"
                    btnInsert.Enabled = False
                    GoTo Error_Trap

                Else
                    lblStatus.Text = "Please open any Assembly"

                End If
            Next
        Else        'If there is no assembly open
            lblStatus.Text = "Please open any Assembly"
        End If

Error_Trap:
        'Handle Errors here
        Exit Sub
    End Sub

    'This Function creates a Block with a Hole in the empty part that is added to the main Assembly
    Private Sub CreateFeatures()

        Dim allPlanes As IADDesignPlanes            ' Holds Design Planes
        Dim refPlane As IADDesignPlane              ' Holds Design Plane
        Dim objPlaneSketch As IADSketch             ' Holds the Reference Sketch
        Dim objADSketchFigures As IADSketchFigures  ' Holds all Sketch Figures
        Dim objFeatures As IADPartFeatures          ' Holds all Part Features
        Dim objExtrudeBossFeature As IADPartFeature ' Holds the Extrusion Feature

        allPlanes = DesignSession.DesignPlanes  ' Get all Planes in the Part
        refPlane = allPlanes.Item("XY-Plane")   ' Get XY Plane

        objPlaneSketch = objPartSession.Sketches.AddSketch(Nothing, refPlane, "Sketch1") 'Add Sketch to XY Plane
        objADSketchFigures = objPlaneSketch.Figures 'Get the Sketch added to XY Plane

        'The following calls sketch a Rectangle and a Circle in the XY Plane
        Call objPlaneSketch.BeginChange()
        Call objPlaneSketch.Figures.AddRectangle(-10, -10, 10, 10)
        Call objPlaneSketch.Figures.AddCircle(0, 0, 5)
        Call objPlaneSketch.EndChange()

        objPlaneSketch = objPartSession.Sketches.Item("Sketch1") ' Name the Sketch as Sketch1
        objFeatures = objPartSession.Features
        'Adds the Extrusion feature using the Sketch created above
        objExtrudeBossFeature = objFeatures.AddExtrudedBoss(objPlaneSketch, 5.0#, _
                            ADPartFeatureEndCondition.AD_MID_PLANE, Nothing, Nothing, _
                            0, ADDirectionType.AD_ALONG_NORMAL, Nothing, _
                            Nothing, False, 0.0#, False, "BlockWithHoleFeature11")

    End Sub

    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        If rootObj Is Nothing Then
            Me.Close()
            Exit Sub
        Else
            rootObj = Nothing
            Me.Close()
        End If
    End Sub

    'This function gets the automation hook for the running instance of Alibre.
    'If there is any Assembly Session open, then the 'Insert part and Save Assembly' button gets enabled.
    'If there is no Assembly Session open, this button remains disabled.
    Private Sub frmPartInsertAssmSave_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        On Error Resume Next

        'Gets the automation hook for the running instance of Alibre
        hook = GetObject(, "AlibreX.AutomationHook")

        If (hook Is Nothing) Then       'If Alibre Design is not initialized
            btnInsert.Enabled = False
            lblStatus.Text = "Open any Assembly in Alibre and restart this application"
        Else                            'If Alibre Design is initialized
            rootObj = hook.Root
            btnInsert.Enabled = True
        End If
    End Sub
End Class
